//
// Hardware.hpp:
//	Interface Definition of classes parallelPort and gloveDriver.
//
// Purpose:
//	Access the parallel port with the proper timings for a power
//	glove.  'parallelPort' provides low-level access to the
//	parallel port, and 'gloveDriver' assigns received data to
//	a GloveData object.  The address of the parallel port is
//	determined when 'parallelPort' is instantiated!  These classes
//	should be easier to use than the previous 'C' version's code.
//	Machine timings are now determined automatically through
//	a somewhat roundabout method.  There seems to be no
//	easy way to do this, I'm afraid.  Building a machine-
//	independent delay of 15 microseconds is no trivial task.
//
// Author(s): Mark Pflaging   C-serve  70233,1552
// Copyright 1992   Mark Thomas Pflaging
//
// Also see authorships in OOPGLOVE.DOC.
// Date:    5/15/92 - 9/21/92
//
#ifndef __HARDWARE_HPP
#define __HARDWARE_HPP

#include <dos.h>

#ifndef __GLOVEDAT_HPP
#include "GloveDat.hpp"
#endif

#ifndef __FINGERS_HPP
#include "Fingers.hpp"
#endif

#ifndef	__ZTIMER_H
#include "ztimer.h"
#endif

#ifndef __ISR_HPP
#include "isr.hpp"
#endif

#ifndef __INI_HPP
#include "ini.hpp"
#endif

const int aone = (int)'1';
typedef enum {
	LPT1 = aone,
	LPT2,
	LPT3
} tPort;

// bits for i/o ports
const unsigned char         GDATA      =     0x10;    // PG data in
const unsigned char         GLATCH     =     0x02;    // PG latch out
const unsigned char         GCLOCK     =     0x01;    // PG clock out
const unsigned char         GCLOLAT    =     0x03;    // clock + latch
const unsigned long         RES        =     6000000L;


class parallelPort {

	int     InPort, OutPort, userOutPort;
	void set_glove_inPort( int ip ) { InPort = ip; }
	void set_glove_outPort( int op ) { OutPort = op; }
public:
	parallelPort(InitFile & ini);
	parallelPort(tPort p);
	setPort(tPort p);

protected:
	unsigned char BITREAD() { return ( ( inportb( InPort ) & GDATA ) >> 4 ); }
	void C0L0() { outportb( OutPort, 0 ); }        // clock 0 latch 0.
	void C0L1() { outportb( OutPort, GLATCH ); }   // clock 0 latch 1.
	void C1L0() { outportb( OutPort, GCLOCK ); }   // clock 1 latch 0.
	void C1L1() { outportb( OutPort, GCLOLAT ); }  // clock 1 latch 1.
};

class machineTimer {
	PZTimer		ptimer;
	int count_once();
	int get_count();
	// N and D will be calibrated.
	unsigned long int Num;
	unsigned long int Den;
	unsigned long int val;
	unsigned long int test_value;

	// The following three members have been moved to the
	// "graveyard" OLD.CPP.  They will probably be deleted.
	static char volatile timer_done;
	// timer function
	static int settimer(unsigned long delay);
	static char times_up();

	// The following *function can be assigned to a
	// function which will be executed in place of the
	// instantiated delay loop.  Function execution time
	// should be approximately the same as the instantiated
	// delay.   -DPRC
	// (Frankly, I would be surprised if you actually
	// found a use for this. -mtp)
	void (*timing_func) (void);

public:
	machineTimer(unsigned int val, unsigned long N, unsigned long D)
		: Num(N), Den (D), val((unsigned long)val) { }
	// calibrates N and D
	machineTimer(unsigned int val, void (*func) (void),
		char * ascN = NULL, char * ascD = NULL);

	// The infamous delay loop!!
	void fdelay();
	void fdelay(int newdelay);
	// Now appearing at area theaters in static form!
	static void fdelay(unsigned long N, unsigned long D,
		unsigned long val);

	void multipliedDelay(int multiplier);
	void test();
};

class gloveDriver : parallelPort, segaISR, public GloveData, public Fingers {
	// delay values for sending and sampling data.
	machineTimer & D2BYTES, & D2BITS, & D2SLOW;
	static gloveDriver * forISR;	// see hardware.cpp
	// # of interrupts to ignore glove so it
	// will recover from mode setup
	int glove_ignore;
	// number of times glove has been not ready
	unsigned unready;
	// number of samples "missed" by the client program.
	unsigned nmissed;
	// Number of MILLISECONDS between samples.
	int glove_tc;
	// Number of milliseconds to go until next sample.
	int glove_pause;

	static char * title;

	unsigned char get_glove_byte();	// read byte from glove
	waitUntilReady();
	static void gloveISR();
	void gloveHandler();

public:
	gloveDriver::gloveDriver(InitFile & ini);
	~gloveDriver() {
		delete &D2BYTES;
		delete &D2BITS;
		delete &D2SLOW;
		C0L0();
	}
	virtual int updateGloveData();		// fill up the structure
	void slowDelay(int i) { D2SLOW.multipliedDelay(i); }

	int yield();
	void waitForSample();
	void test();

protected:
	// Returns 0 if not ready.  Delay repeat calls by 2-4 ms
	int is_glove_ready();		   // is glove ready?

	void set_glove_hiResMode();        // puts glove in hires mode
};

#endif
